#!/bin/bash
shopt -s compat32

################################################################################
# Name: compEpoch                                                              # 
# Purpose: compare Epoch fields numerically, and the package                   # 
#          with the larger number is considered newer.                         # 
# Return code: 0(Equal), 1(1st package is newer), 2(2nd package is newer)      # 
# Requirement: 1st argument must be the full package name with version/release #
#              2nd argument must be just package name                          #
################################################################################
compEpoch() {
  epochOne=$(rpm -qp --qf "%{epoch}" $1)
  epochTwo=$(rpm -q --qf "%{epoch}" $2)

  if [[ $epochOne =~ '(none)' ]]; then
    epochOne=0
  fi
  if [[ $epochTwo =~ '(none)' ]]; then
    epochTwo=0
  fi

  if [[ $epochOne -gt $epochTwo ]]; then
    return 1
  elif [[ $epochOne -lt $epochTwo ]]; then
    return 2
  else
    return 0
  fi
}

################################################################################
# Name: compVerRel                                                             # 
# Purpose: compare version or release fields, and the larger one wins.         # 
# Return code: 0(Equal), 1(1st package is newer), 2(2nd package is newer)      # 
################################################################################
compVerRel() {
#  echo "IP Release:$1  |  PI Release:$2"
  convToArray $1
  first_arr=(${rst_arr[*]})
#  echo "IP array: ${first_arr[*]}"
  convToArray $2
  second_arr=(${rst_arr[*]})
#  echo "PI array: ${second_arr[*]}"

  if [ ${#first_arr[@]} -gt ${#second_arr[@]} ]; then
    length=${#second_arr[@]}
    rst=1
  elif [ ${#first_arr[@]} -lt ${#second_arr[@]} ]; then
    length=${#first_arr[@]}
    rst=2
  else
    length=${#first_arr[@]}
    rst=0
  fi

  # compare arrays
  for ((i=0;i<length;i++))
    do
      elm1=${first_arr[i]}
      elm2=${second_arr[i]}
      
      echo "$elm1" | grep -q '[0-9]'
      numflag1=$?
      echo "$elm2" | grep -q '[0-9]'
      numflag2=$?
      if [ $numflag1 = 0 ]; then
        if [ $numflag2 != 0 ]; then
          return 1
        else
          if [ $elm1 -gt $elm2 ]; then
            return 1
          elif [ $elm1 -lt $elm2 ]; then
            return 2 
          else
            continue
          fi
        fi
      else
        if [ $numflag2 = 0 ]; then
          return 2 
        else # string compare
          if [ $elm1 = $elm2 ]; then
            continue 
          fi  
          if [ $elm1 > $elm2 ]; then
            return 1
          fi 
          if [ $elm1 <  $elm2 ]; then
            return 2
          fi 
        fi
      fi
    done

  return $rst
}

################################################################################
# Name: convToArray                                                            # 
# Purpose: compare version or release fields into array.                       # 
################################################################################
convToArray() {
  #string="2xFg33.+f.5"
  numeric_re='^[0-9]+$'
  alpha_re='^[a-zA-Z]+$'

  unset rst_arr
  # remove non-alphanumeric characters
  #array=(${string//[^0-9a-zA-Z]/ })
  array=(${1//[^0-9a-zA-Z]/ })

  for i in "${!array[@]}"
  do
    #echo "$i=>${array[i]}"
    if [[ ${array[i]} =~ $numeric_re ]]; then
      #echo "numeric"
      rst_arr=(${rst_arr[*]} ${array[i]})
    elif [[ ${array[i]} =~ $alpha_re ]]; then
      #echo "alpha"
      rst_arr=(${rst_arr[*]} ${array[i]})
    else
      arr_elem=${array[i]}
      tmp_elem=''
      flag=-1 # 0=>num 1=>alpha
      for (( j=0;j<${#arr_elem};j++ ))
        do
          ch=${arr_elem:j:1}
          echo "$ch" | grep -q '[0-9]'
          if [ $? = 0 ]; then
            if [ $flag = 0 ]; then
              tmp_elem=$tmp_elem$ch
            elif [ $flag = 1 ]; then
              rst_arr=(${rst_arr[*]} $tmp_elem)
              tmp_elem=$ch
              flag=0
            else
              tmp_elem=$ch
              flag=0;
            fi
          else
            if [ $flag = 1 ]; then
              tmp_elem=$tmp_elem$ch
            elif [ $flag = 0 ]; then
              rst_arr=(${rst_arr[*]} $tmp_elem)
              tmp_elem=$ch
              flag=1
            else
              tmp_elem=$ch
              flag=1;
            fi
          fi
        done
      rst_arr=(${rst_arr[*]} $tmp_elem)
    fi
  done
}

################################################################################
# Name: summary                                                                # 
# Purpose: print out the summary of rpm packages in the system                 # 
################################################################################
summary() {
  echo "*********************************"
  echo "*            SUMMARY            *"
  echo "*********************************"
  echo "Total RPM to be installed : $totalRPM"
  echo "Total Installed RPM : $(($totalRPM-$totalFailedRPM))"
  echo "Total Failure RPM: $totalFailedRPM" 
  echo ""
  echo "    List of Failure RPM"
  for i in ${totalFailedRPM_arr[*]}
    do
      echo "$i"
    done 
}

FILES=*.rpm
RPMS=''
NAME_RE="Name.*: ([^ ]*).*Relocations"
VERSION_RE="Version.*: ([^ ]*).*Vendor"
RELEASE_RE="Release.*: ([^ ]*).*Build Date"
rst_arr=()

# summary variables
totalRPM=0
totalFailedRPM=0
totalFailedRPM_arr=()

#for frpm in $FILES
#  do
#    RPMS=$frpm" "$RPMS
#  done

#yum install -y $RPMS

  for frpm in $FILES
    do
      totalRPM=$(($totalRPM+1))
      compFlag=0 # decide whether compare rpm packages
      # get info regarding rpm package to be installed
      #echo "******************************************"
      #echo "* $frpm"
      #echo "******************************************"
      rpm -qpi $frpm > "rpm_pi.info"
      while read -r line; do
        if [[ $line =~ $NAME_RE ]]; then
          #echo "FOUND PI: ${BASH_REMATCH[1]}"
          rpmName_pi=${BASH_REMATCH[1]}
        fi        
        
        if [[ $line =~ $VERSION_RE ]]; then
          #echo "FOUND PI: ${BASH_REMATCH[1]}"
          rpmVersion_pi=${BASH_REMATCH[1]}
        fi        

        if [[ $line =~ $RELEASE_RE ]]; then
          #echo "FOUND PI: ${BASH_REMATCH[1]}"
          rpmRelease_pi=${BASH_REMATCH[1]}
        fi        
      done < "rpm_pi.info"

      # get info regarding already installed package
      rpm -qi $rpmName_pi > "rpm_i.info"
      while read -r line; do
        # Package is not installed
        if [[ $line =~ "not installed" ]]; then
          compFlag=1
          echo "Error: The package $frpm is not installed."
          totalFailedRPM=$(($totalFailedRPM+1))
          totalFailedRPM_arr=(${totalFailedRPM_arr[*]} $frpm)
        fi

        if [[ $line =~ $NAME_RE ]]; then
          #echo "FOUND IP: ${BASH_REMATCH[1]}"
          rpmName_i=${BASH_REMATCH[1]}
        fi        
        
        if [[ $line =~ $VERSION_RE ]]; then
          #echo "FOUND IP: ${BASH_REMATCH[1]}"
          rpmVersion_i=${BASH_REMATCH[1]}
        fi        

        if [[ $line =~ $RELEASE_RE ]]; then
          #echo "FOUND IP: ${BASH_REMATCH[1]}"
          rpmRelease_i=${BASH_REMATCH[1]}
        fi        
      done < "rpm_i.info"

      if [[ $compFlag == 0 ]]; then
        compEpoch $frpm  $rpmName_i
        compRst=$?

        if [ $compRst == 1 ]; then
          echo "Error: The package $frpm is newer than the already installed package in system"
          echo "       But, the newer package is not installed successfully."
          totalFailedRPM=$(($totalFailedRPM+1))
          totalFailedRPM_arr=(${totalFailedRPM_arr[*]} $frpm)
          continue 
        elif [ $compRst == 2 ]; then
          continue
        fi

        compVerRel $rpmVersion_pi $rpmVersion_i
        compRst=$?
        if [ $compRst == 1 ]; then
          echo "Error: The package $frpm is newer than the already installed package in system"
          echo "       But, the newer package is not installed successfully."
          totalFailedRPM=$(($totalFailedRPM+1))
          totalFailedRPM_arr=(${totalFailedRPM_arr[*]} $frpm)
          continue
        elif [ $compRst == 2 ]; then 
          continue 
        fi

        compVerRel $rpmRelease_pi $rpmRelease_i
        compRst=$?
        if [ $compRst == 1 ]; then
          echo "Error: The package $frpm is newer than the already installed package in system"
          echo "       But, the newer package is not installed successfully."
          totalFailedRPM=$(($totalFailedRPM+1))
          totalFailedRPM_arr=(${totalFailedRPM[*]_arr} $frpm)
        fi
      fi
      
    done

summary

rm -rf rpm_pi.info
rm -rf rpm_i.info
